"""
menu
===============================================================================
"""
from __future__ import annotations
from typing import TYPE_CHECKING

import logging
import os

import wx
from booleanOperations import BooleanOperationManager as Boolean

from wbBase.scripting import MacroMenu
from wbDefcon import Anchor, Component, Point, Guideline, Glyph
from wbDefcon.pens import ContourHit

from ...dialog.anchorDialog import AnchorDialog
from ...dialog.componentDialog import ComponentDialog
from ...dialog.findGlyphDialog import FindGlyphDialog
from ...dialog.guidelineDialog import GuidelineDialog
from ...glyphCommand.commandListDialog import CommandListDialog
from .drawing import LayerPlanes

if TYPE_CHECKING:
    from wbBase.application import App
log = logging.getLogger(__name__)

ID = wx.ID_ANY

getBitmap = lambda name: wx.ArtProvider.GetBitmap(name, wx.ART_MENU, (16, 16))


def makeGlyphEditMacroMenu():
    app:App = wx.GetApp()
    cfg = app.config
    macroFolderPath = [
        os.path.join(
            cfg.Read("/Application/SharedData/Dir", app.sharedDataDir),
            "Macro",
            "_system",
            "_glyph",
        ),
        os.path.join(
            cfg.Read("/Application/PrivateData/Dir", app.privateDataDir),
            "Macro",
            "_system",
            "_glyph",
        ),
    ]
    if any(os.path.isdir(p) for p in macroFolderPath):
        return MacroMenu(folderList=macroFolderPath)


glyphEditMacroMenu = makeGlyphEditMacroMenu()


class DefaultEditMenu(wx.Menu):
    def __init__(self, canvas, subject):
        super(DefaultEditMenu, self).__init__()
        self.canvas = canvas
        self.subject = subject
        self.x = None
        self.y = None

        # Ruler --------------------
        item = wx.MenuItem(self, ID, "Show Ruler", "", wx.ITEM_CHECK)
        self.Append(item)
        item.Check(self.canvas.Parent.rulerShown)
        self.Bind(wx.EVT_MENU, self.on_showRuler, id=item.GetId())

        # Note --------------------
        item = wx.MenuItem(self, ID, "Edit Note", "", wx.ITEM_NORMAL)
        item.SetBitmap(getBitmap("NOTE"))
        self.Append(item)
        self.Bind(wx.EVT_MENU, self.on_editNote, id=item.GetId())

        # Command List --------------------
        item = wx.MenuItem(self, ID, "Command List ...", "", wx.ITEM_NORMAL)
        item.SetBitmap(getBitmap("COMMAND_LIST"))
        self.Append(item)
        self.Bind(wx.EVT_MENU, self.on_command_list, id=item.GetId())

        # Anchors --------------------
        self.AppendSeparator()
        if isinstance(self.subject, Anchor):
            # edit
            item = wx.MenuItem(self, ID, "Edit Anchor", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_editAnchor, id=item.GetId())
            # delete
            item = wx.MenuItem(self, ID, "Delete Anchor", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_deleteAnchor, id=item.GetId())
        else:
            # add new
            item = wx.MenuItem(self, ID, "Add Anchor", "", wx.ITEM_NORMAL)
            item.SetBitmap(getBitmap("ANCHOR_ADD"))
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_addAnchor, id=item.GetId())
            if len(self.glyph.anchors) > 0:
                # clear Anchors
                clearAnchors = wx.MenuItem(
                    self, ID, "Delete All Anchors", "", wx.ITEM_NORMAL
                )
                self.Append(clearAnchors)
                self.Bind(wx.EVT_MENU, self.on_clearAnchors, id=clearAnchors.GetId())

        # Guidelines --------------------
        self.AppendSeparator()
        if isinstance(self.subject, Guideline):
            # edit
            item = wx.MenuItem(self, ID, "Edit Guideline", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_editGuideline, id=item.GetId())
            # delete
            item = wx.MenuItem(self, ID, "Delete Guideline", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_deleteGuideline, id=item.GetId())
            if isinstance(self.subject.glyph, Glyph):
                # convert to global guide
                item = wx.MenuItem(self, ID, "Convert to global Guideline", "", wx.ITEM_NORMAL)
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_convertToGlobalGuideline, id=item.GetId())
        else:
            # add new
            item = wx.MenuItem(self, ID, "Add Guideline", "", wx.ITEM_NORMAL)
            item.SetBitmap(getBitmap("GUIDELINE_ADD"))
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_addGuideline, id=item.GetId())
            if len(self.glyph.guidelines) > 0:
                # clear Guidelines
                clearGuidelines = wx.MenuItem(
                    self, ID, "Delete All Guidelines", "", wx.ITEM_NORMAL
                )
                self.Append(clearGuidelines)
                self.Bind(wx.EVT_MENU, self.on_clearGuidelines, id=clearGuidelines.GetId())

        # Components --------------------
        self.AppendSeparator()
        # add new
        item = wx.MenuItem(self, ID, "Add Component", "", wx.ITEM_NORMAL)
        item.SetBitmap(getBitmap("COMPONENT_ADD"))
        self.Append(item)
        self.Bind(wx.EVT_MENU, self.on_addComponent, id=item.GetId())
        if isinstance(self.subject, Component):
            # edit
            item = wx.MenuItem(self, ID, "Edit Component", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_editComponent, id=item.GetId())
            # properties
            item = wx.MenuItem(self, ID, "Component Properties ...", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_componentProperties, id=item.GetId())
            # set index
            item = wx.MenuItem(self, ID, "Set Component Index", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_setComponentIndex, id=item.GetId())
            # decompose
            item = wx.MenuItem(self, ID, "Decompose Component", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_decomposeComponent, id=item.GetId())
            # delete
            item = wx.MenuItem(self, ID, "Remove Component", "", wx.ITEM_NORMAL)
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_deleteComponent, id=item.GetId())
        else:
            if len(self.glyph.components) > 0:
                # decompose all
                item = wx.MenuItem(
                    self, ID, "Decompose All Components", "", wx.ITEM_NORMAL
                )
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_decomposeAllComponents, id=item.GetId())
                # clear Components
                item = wx.MenuItem(
                    self, ID, "Remove All Components", "", wx.ITEM_NORMAL
                )
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_clearComponents, id=item.GetId())

        # Points --------------------
        if isinstance(self.subject, Point):
            self.AppendSeparator()
            if self.subject.segmentType is not None:
                # Set Startpoint
                item = wx.MenuItem(self, ID, "Set Startpoint", "", wx.ITEM_NORMAL)
                item.SetBitmap(getBitmap("SET_STARTPOINT"))
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_setStartPoint, id=item.GetId())
                # set smooth

        # Contours --------------------
        if len(self.glyph) > 0:
            self.AppendSeparator()
            item = wx.MenuItem(self, ID, "Reverse All Contours", "", wx.ITEM_NORMAL)
            item.SetBitmap(getBitmap("REVERSE_ALL_CONTOURS"))
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_reverseAllContours, id=item.GetId())
            if isinstance(self.subject, (Point, ContourHit)):
                # change direction
                item = wx.MenuItem(self, ID, "Clockwise Direction", "", wx.ITEM_RADIO)
                self.Append(item)

                item = wx.MenuItem(
                    self, ID, "Counter-Clockwise Direction", "", wx.ITEM_RADIO
                )
                self.Append(item)
                # set index
                item = wx.MenuItem(self, ID, "Set Contour Index", "", wx.ITEM_NORMAL)
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_setContourIndex, id=item.GetId())
                # substract
                item = wx.MenuItem(self, ID, "Substract Contour", "", wx.ITEM_NORMAL)
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_substractContour, id=item.GetId())
                # get intersection
                item = wx.MenuItem(
                    self, ID, "Get Intersection with Contour", "", wx.ITEM_NORMAL
                )
                item.SetBitmap(getBitmap("GET_INTERSECTION"))
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_getIntersection, id=item.GetId())
                # remove intersection
                item = wx.MenuItem(
                    self, ID, "Remove Intersection with Contour", "", wx.ITEM_NORMAL
                )
                item.SetBitmap(getBitmap("DELETE_INTERSECTION"))
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_removeIntersection, id=item.GetId())

            item = wx.MenuItem(self, ID, "Remove Overlap", "", wx.ITEM_NORMAL)
            item.SetBitmap(getBitmap("MERGE_CONTOURS"))
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_removeOverlap, id=item.GetId())
        # Macro Menu -------------------------------------
        if glyphEditMacroMenu:
            item = self.AppendSubMenu(glyphEditMacroMenu, "Macros")
            item.SetBitmap(wx.ArtProvider.GetBitmap("PYTHON", wx.ART_MENU, (16, 16)))
            item.SetFont(wx.GetApp().TopWindow.Font)

        # cfg = self.app.TopWindow.config
        # macroFolderPath = [
        #     os.path.join(
        #         cfg.Read("/Application/SharedData/Dir", self.app.sharedDataDir),
        #         "Macro",
        #         "_system",
        #         "_glyph",
        #     ),
        #     os.path.join(
        #         cfg.Read("/Application/PrivateData/Dir", self.app.privateDataDir),
        #         "Macro",
        #         "_system",
        #         "_glyph",
        #     ),
        # ]
        # if any(os.path.isdir(p) for p in macroFolderPath):
        #     macroMenu = MacroMenu(folderList=macroFolderPath)
        #     item = self.AppendSubMenu(macroMenu, "Macros")
        #     item.SetBitmap(wx.ArtProvider.GetBitmap("PYTHON", wx.ART_MENU, (16, 16)))
        #     item.SetFont(wx.GetApp().TopWindow.Font)

    @property
    def app(self):
        return wx.GetApp()

    @property
    def glyph(self):
        planeStack = self.canvas.getActiveDrawingPlaneStack()
        if isinstance(planeStack, LayerPlanes):
            return planeStack.glyph
        else:
            return self.canvas.glyph

    @property
    def font(self):
        glyph = self.glyph
        if glyph is not None:
            return glyph.font

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_showRuler(self, event):
        self.canvas.Parent.showRuler(not self.canvas.Parent.rulerShown)

    def on_editNote(self, event):
        with wx.TextEntryDialog(
            self.canvas,
            'Glyph "%s" on layer "%s"' % (self.glyph.name, self.glyph.layer.name),
            "Glyph Note",
            self.glyph.note or "",
            wx.TextEntryDialogStyle | wx.TE_MULTILINE,
            wx.DefaultPosition,
        ) as noteDialog:
            if noteDialog.ShowModal() == wx.ID_OK:
                self.glyph.note = noteDialog.GetValue()

    def on_command_list(self, event):
        with CommandListDialog() as dialog:
            dialog.choice_target.SetSelection(0)  # current glyph
            if dialog.ShowModal() == wx.ID_OK:
                commandList = dialog.commandList
                if commandList:
                    dialog.executeCommandList()


    # Anchors --------------------

    def on_editAnchor(self, event):
        anchor = self.subject
        anchorDialog = AnchorDialog(self.canvas, anchor.asDict())
        if anchorDialog.ShowModal() == wx.ID_OK:
            d = anchorDialog.anchorDict
            anchor.x = d["x"]
            anchor.y = d["y"]
            anchor.name = d["name"]
            anchor.color = d["color"]
        anchorDialog.Destroy()

    def on_deleteAnchor(self, event):
        self.glyph.removeAnchor(self.subject)

    def on_addAnchor(self, event):
        with AnchorDialog(self.canvas) as anchorDialog:
            anchorDialog.spinCtrl_x.Value = round(self.x)
            anchorDialog.spinCtrl_y.Value = round(self.y)
            if anchorDialog.ShowModal() == wx.ID_OK:
                self.glyph.appendAnchor(anchorDialog.anchorDict)

    def on_clearAnchors(self, event):
        self.glyph.clearAnchors()


    # Guidelines --------------------

    def on_editGuideline(self, event):
        guideline = self.subject
        with GuidelineDialog(self.canvas, guideline.getDataForSerialization()) as guidelineDialog:
            if guidelineDialog.ShowModal() == wx.ID_OK:
                d = guidelineDialog.guidelineDict
                guideline.x = d["x"]
                guideline.y = d["y"]
                guideline.name = d["name"]
                guideline.color = d["color"]

    def on_deleteGuideline(self, event):
        if self.subject.glyph is not None:
            self.glyph.removeGuideline(self.subject)
        else:
            self.font.removeGuideline(self.subject)

    def on_convertToGlobalGuideline(self, event):
        guideline = self.subject
        guidelineDict = guideline.getDataForSerialization()
        self.font.appendGuideline(guidelineDict)
        self.glyph.removeGuideline(self.subject)


    def on_addGuideline(self, event):
        with GuidelineDialog(self.canvas) as guidelineDialog:
            guidelineDialog.spinCtrl_x.Value = round(self.x)
            guidelineDialog.spinCtrl_y.Value = round(self.y)
            if guidelineDialog.ShowModal() == wx.ID_OK:
                self.glyph.appendGuideline(guidelineDialog.guidelineDict)

    def on_clearGuidelines(self, event):
        self.glyph.clearGuidelines()

    # Components --------------------

    def on_editComponent(self, event):
        self.subject.font[self.subject.baseGlyph].show()

    def on_componentProperties(self, event):
        component = self.subject
        componentDialog = ComponentDialog(self.canvas, component)
        if componentDialog.ShowModal() == wx.ID_OK:
            component.baseGlyph = componentDialog.baseGlyph
            component.offset = componentDialog.offset
            component.scale = componentDialog.scale
        componentDialog.Destroy()

    def on_setComponentIndex(self, event):
        font = self.font
        if font:
            font.holdNotifications()
        glyph = self.glyph
        glyph.disableNotifications()
        glyph.undoManager.saveState()
        pointedComponent = self.subject
        pointedComponentIndex = glyph.componentIndex(pointedComponent)
        with wx.SingleChoiceDialog(
            self.Parent,
            "Select new Component Index",
            "Component Index",
            [str(i) for i in range(len(glyph.components))],
        ) as dialog:
            dialog.Selection = pointedComponentIndex
            if dialog.ShowModal() == wx.ID_OK:
                newComponentIndex = dialog.Selection
                if newComponentIndex != pointedComponentIndex:
                    glyph.removeComponent(pointedComponent)
                    glyph.insertComponent(newComponentIndex, pointedComponent)
        glyph.enableNotifications()
        glyph.postNotification("Glyph.ComponentsChanged")
        if font:
            font.releaseHeldNotifications()


    def on_decomposeComponent(self, event):
        self.glyph.decomposeComponent(self.subject)

    def on_deleteComponent(self, event):
        self.glyph.removeComponent(self.subject)

    def on_addComponent(self, event):
        findGlyphDialog = FindGlyphDialog(self.canvas, self.glyph.layer)
        # findGlyphDialog.layer = self.glyph.layer
        if (
            findGlyphDialog.ShowModal() == wx.ID_OK
            and findGlyphDialog.selctedGlyph is not None
        ):
            component = self.glyph.instantiateComponent()
            component.baseGlyph = findGlyphDialog.selctedGlyph
            self.glyph.appendComponent(component)
        findGlyphDialog.Destroy()

    def on_decomposeAllComponents(self, event):
        self.glyph.decomposeAllComponents()

    def on_clearComponents(self, event):
        self.glyph.clearComponents()

    # Points --------------------

    def on_setStartPoint(self, event):
        for contour in self.glyph:
            if self.subject in contour:
                contour.setStartPoint(contour.index(self.subject))

    # Contours --------------------

    def on_reverseAllContours(self, event):
        for contour in self.glyph:
            contour.reverse()

    def on_setContourIndex(self, event):
        font = self.font
        if font:
            font.holdNotifications()
        glyph = self.glyph
        glyph.disableNotifications()
        glyph.undoManager.saveState()
        pointedContour = None
        if isinstance(self.subject, ContourHit):
            pointedContour = self.subject.contour
            pointedContourIndex = self.subject.contourIndex
        else:
            for contourIndex, contour in enumerate(glyph):
                if self.subject in contour:
                    pointedContour = contour
                    pointedContourIndex = contourIndex
                    break
        if pointedContour:
            with wx.SingleChoiceDialog(
                self.Parent,
                "Select new Contour Index",
                "Contour Index",
                [str(i) for i in range(len(glyph))],
            ) as dialog:
                dialog.Selection = pointedContourIndex
                if dialog.ShowModal() == wx.ID_OK:
                    newContourIndex = dialog.Selection
                    if newContourIndex != pointedContourIndex:
                        glyph.removeContour(pointedContour)
                        glyph.insertContour(newContourIndex, pointedContour)
        glyph.enableNotifications()
        glyph.postNotification("Glyph.ContoursChanged")
        if font:
            font.releaseHeldNotifications()

    def on_substractContour(self, event):
        font = self.font
        if font:
            font.holdNotifications()
        glyph = self.glyph
        contours = [c for c in glyph]
        glyph.disableNotifications()
        glyph.undoManager.saveState()
        glyph.clearContours()
        Boolean.difference(
            [c for c in contours if self.subject not in c],
            [c for c in contours if self.subject in c],
            glyph.getPointPen(),
        )
        glyph.round(roundAnchors=False, roundComponents=False)
        glyph.enableNotifications()
        glyph.postNotification("Glyph.ContoursChanged")
        if font:
            font.releaseHeldNotifications()

    def on_getIntersection(self, event):
        font = self.font
        if font:
            font.holdNotifications()
        glyph = self.glyph
        contours = list(glyph)
        glyph.disableNotifications()
        glyph.undoManager.saveState()
        glyph.clearContours()
        Boolean.intersection(
            [c for c in contours if self.subject not in c],
            [c for c in contours if self.subject in c],
            glyph.getPointPen(),
        )
        glyph.round(roundAnchors=False, roundComponents=False)
        glyph.enableNotifications()
        glyph.postNotification("Glyph.ContoursChanged")
        if font:
            font.releaseHeldNotifications()

    def on_removeIntersection(self, event):
        font = self.font
        if font:
            font.holdNotifications()
        glyph = self.glyph
        contours = [c for c in glyph]
        glyph.disableNotifications()
        glyph.undoManager.saveState()
        glyph.clearContours()
        Boolean.xor(
            [c for c in contours if self.subject not in c],
            [c for c in contours if self.subject in c],
            glyph.getPointPen(),
        )
        glyph.round(roundAnchors=False, roundComponents=False)
        glyph.enableNotifications()
        glyph.postNotification("Glyph.ContoursChanged")
        if font:
            font.releaseHeldNotifications()

    def on_removeOverlap(self, event):
        font = self.font
        if font:
            font.holdNotifications()
        glyph = self.glyph
        contours = [c for c in glyph]
        glyph.disableNotifications()
        glyph.clearContours()
        glyph.undoManager.saveState()
        Boolean.union(contours, glyph.getPointPen())
        glyph.round(roundAnchors=False, roundComponents=False)
        glyph.enableNotifications()
        glyph.postNotification("Glyph.ContoursChanged")
        if font:
            font.releaseHeldNotifications()



"""
template
===============================================================================

Document Templates for UFOs and imports to UFO WorkBench
"""
import os

import wx
from fontTools.ufoLib import UFOFileStructure
from glyphsLib.builder import to_ufos
from glyphsLib.parser import load

import wbDefcon
from wbBase.document import dbg
from wbBase.document.template import (
    DEFAULT_TEMPLATE_FLAGS,
    TEMPLATE_NO_CREATE,
    DocumentTemplate,
)

from .document import UfoDocument
from .view.font import UfoFontView


class UfoTemplate(DocumentTemplate):
    """
    Main Template for Unified Font Objects
    """
    def __init__(self, manager):
        DocumentTemplate.__init__(
            self,
            manager=manager,
            description="UFO (Unified Font Object)",
            filter="*.ufo;*.ufoz",
            dir="",
            ext=".ufo",
            docTypeName="UFO document",
            viewTypeName="UFO Font View",
            docType=UfoDocument,
            viewType=UfoFontView,
            flags=DEFAULT_TEMPLATE_FLAGS,
            icon=wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_FRAME_ICON),
        )


class VFBTemplate(DocumentTemplate):
    """Template for FontLab VFB files - import only"""

    def __init__(self, manager):
        DocumentTemplate.__init__(
            self,
            manager=manager,
            description="FontLab file (import only)",
            filter="*.vfb",
            dir="",
            ext=".ufo",
            docTypeName="UFO document",
            viewTypeName="UFO Font View",
            docType=UfoDocument,
            viewType=UfoFontView,
            flags=DEFAULT_TEMPLATE_FLAGS | TEMPLATE_NO_CREATE,
            icon=wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_FRAME_ICON),
        )


class GlyphsTemplate(DocumentTemplate):
    """Template for Glyphs App files - import only"""

    def __init__(self, manager):
        DocumentTemplate.__init__(
            self,
            manager=manager,
            description="Glyphs App file (import only)",
            filter="*.glyphs",
            dir="",
            ext=".ufo",
            docTypeName="UFO document",
            viewTypeName="UFO Font View",
            docType=UfoDocument,
            viewType=UfoFontView,
            flags=DEFAULT_TEMPLATE_FLAGS | TEMPLATE_NO_CREATE,
            icon=wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_FRAME_ICON),
        )

    def CreateDocument(self, path, flags):
        """
        Creates a new instance of the associated document class (UfoDocument)
        for each master in the Glyphs file
        """
        dbg(f'GlyphsTemplate.CreateDocument(path="{path}", flags={flags})', indent=1)
        result = None
        if os.path.isfile(path):
            with open(path, "r", encoding="utf-8") as glyphsFile:
                with wx.BusyInfo("Loading Glyphs File please wait"):
                    glyphsFont = load(glyphsFile)
                    ufos = to_ufos(glyphsFont, ufo_module=wbDefcon)
            result = []
            for font in ufos:
                font._ufoFormatVersion = 3
                font._ufoFileStructure = UFOFileStructure.ZIP
                doc = self._docType()
                doc.template = self.documentManager.FindTemplateByType(UfoTemplate)
                self.documentManager.AddDocument(doc)
                doc.font = font
                if doc.OnCreate(path, flags):
                    doc.set_modificationDate()
                    doc.saved = False
                    doc.modified = False
                    doc.UpdateAllViews(self, ("font loaded", True))
                    result.append(doc)
                else:
                    if doc in self.documentManager.documents:
                        doc.DeleteAllViews()
        dbg(f"GlyphsTemplate.CreateDocument() returns {result}", indent=0)
        return result



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

from .commandListCtrl import CommandListCtrl
import wx
import wx.xrc
import wx.propgrid as pg

###########################################################################
## Class CommandListDialogUI
###########################################################################

class CommandListDialogUI ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Command List", pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        sizer_main = wx.BoxSizer( wx.VERTICAL )

        sizer_head = wx.GridBagSizer( 0, 0 )
        sizer_head.SetFlexibleDirection( wx.BOTH )
        sizer_head.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.label_target = wx.StaticText( self, wx.ID_ANY, u"Apply command list to", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_target.Wrap( -1 )

        sizer_head.Add( self.label_target, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_targetChoices = [ u"Current Glyph", u"Selected Glyphs in Current Font", u"All Glyphs in Current Font", u"All Glyphs in all open Fonts", u"Glyphs from List in Current Font", u"Glyphs from List in all open Fonts" ]
        self.choice_target = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_targetChoices, 0 )
        self.choice_target.SetSelection( 0 )
        sizer_head.Add( self.choice_target, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.message_target = wx.StaticText( self, wx.ID_ANY, u"Action set will affect some glyphs in some fonts", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.message_target.Wrap( -1 )

        sizer_head.Add( self.message_target, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 2 ), wx.ALL, 5 )

        self.labet_saved_lists = wx.StaticText( self, wx.ID_ANY, u"Saved command lists", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.labet_saved_lists.Wrap( -1 )

        sizer_head.Add( self.labet_saved_lists, wx.GBPosition( 2, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_saved_listsChoices = []
        self.choice_saved_lists = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_saved_listsChoices, 0 )
        self.choice_saved_lists.SetSelection( 0 )
        sizer_head.Add( self.choice_saved_lists, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizer_head.AddGrowableCol( 1 )

        sizer_main.Add( sizer_head, 0, wx.EXPAND, 5 )

        self.staticline_head = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        sizer_main.Add( self.staticline_head, 0, wx.BOTTOM|wx.EXPAND|wx.TOP, 5 )

        sizer_command = wx.BoxSizer( wx.HORIZONTAL )

        sizer_commands_available = wx.BoxSizer( wx.VERTICAL )

        self.label_commands_available = wx.StaticText( self, wx.ID_ANY, u"Available commands", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_commands_available.Wrap( -1 )

        sizer_commands_available.Add( self.label_commands_available, 0, wx.ALL, 5 )

        self.treeCtrl_commands_available = wx.TreeCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TR_DEFAULT_STYLE|wx.TR_HIDE_ROOT|wx.TR_TWIST_BUTTONS )
        sizer_commands_available.Add( self.treeCtrl_commands_available, 1, wx.ALL|wx.EXPAND, 5 )


        sizer_command.Add( sizer_commands_available, 1, wx.EXPAND, 5 )

        sizer_command_select = wx.BoxSizer( wx.VERTICAL )


        sizer_command_select.Add( ( 0, 20), 1, wx.EXPAND, 5 )

        self.button_add = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_add.SetBitmap( wx.ArtProvider.GetBitmap( 'GO_FORWARD', wx.ART_BUTTON ) )
        self.button_add.SetBitmapDisabled( wx.ArtProvider.GetBitmap( 'GO_FORWARD_DISABLED', wx.ART_BUTTON ) )
        self.button_add.Enable( False )
        self.button_add.SetToolTip( u"Add command to list" )

        sizer_command_select.Add( self.button_add, 0, wx.ALL, 5 )

        self.button_remove = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_remove.SetBitmap( wx.ArtProvider.GetBitmap( 'GO_BACK', wx.ART_BUTTON ) )
        self.button_remove.SetBitmapDisabled( wx.ArtProvider.GetBitmap( 'GO_BACK_DISABLED', wx.ART_BUTTON ) )
        self.button_remove.Enable( False )
        self.button_remove.SetToolTip( u"Remove command from list" )

        sizer_command_select.Add( self.button_remove, 0, wx.ALL, 5 )

        self.button_open = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_open.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_FILE_OPEN, wx.ART_BUTTON ) )
        self.button_open.SetToolTip( u"Open command list from file" )

        sizer_command_select.Add( self.button_open, 0, wx.ALL, 5 )

        self.button_save = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_save.SetBitmap( wx.ArtProvider.GetBitmap( 'FILE_SAVE', wx.ART_BUTTON ) )
        self.button_save.SetBitmapDisabled( wx.ArtProvider.GetBitmap( 'FILE_SAVE_DISABLED', wx.ART_BUTTON ) )
        self.button_save.Enable( False )
        self.button_save.SetToolTip( u"Save command list to file" )

        sizer_command_select.Add( self.button_save, 0, wx.ALL, 5 )


        sizer_command.Add( sizer_command_select, 0, wx.EXPAND, 5 )

        sizer_command_list = wx.BoxSizer( wx.VERTICAL )

        self.label_command_list = wx.StaticText( self, wx.ID_ANY, u"Command list", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_command_list.Wrap( -1 )

        sizer_command_list.Add( self.label_command_list, 0, wx.ALL, 5 )

        self.listCtrl_commands = CommandListCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LC_NO_HEADER|wx.LC_REPORT|wx.LC_SINGLE_SEL|wx.LC_VIRTUAL )
        sizer_command_list.Add( self.listCtrl_commands, 1, wx.ALL|wx.EXPAND, 5 )


        sizer_command.Add( sizer_command_list, 1, wx.EXPAND, 5 )

        sizer_command_sort = wx.BoxSizer( wx.VERTICAL )


        sizer_command_sort.Add( ( 0, 20), 1, wx.EXPAND, 5 )

        self.button_up = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_up.SetBitmap( wx.ArtProvider.GetBitmap( 'GO_UP', wx.ART_BUTTON ) )
        self.button_up.SetBitmapDisabled( wx.ArtProvider.GetBitmap( 'GO_UP_DISABLED', wx.ART_BUTTON ) )
        self.button_up.Enable( False )
        self.button_up.SetToolTip( u"Move selected command up" )

        sizer_command_sort.Add( self.button_up, 0, wx.ALL, 5 )

        self.button_down = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_down.SetBitmap( wx.ArtProvider.GetBitmap( 'GO_DOWN', wx.ART_BUTTON ) )
        self.button_down.SetBitmapDisabled( wx.ArtProvider.GetBitmap( 'GO_DOWN_DISABLED', wx.ART_BUTTON ) )
        self.button_down.Enable( False )
        self.button_down.SetToolTip( u"Move selected command down" )

        sizer_command_sort.Add( self.button_down, 0, wx.ALL, 5 )

        self.button_clear = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_clear.SetBitmap( wx.ArtProvider.GetBitmap( 'DELETE', wx.ART_BUTTON ) )
        self.button_clear.SetBitmapDisabled( wx.ArtProvider.GetBitmap( 'DELETE_DISABLED', wx.ART_BUTTON ) )
        self.button_clear.Enable( False )
        self.button_clear.SetToolTip( u"Clear command list" )

        sizer_command_sort.Add( self.button_clear, 0, wx.ALL, 5 )


        sizer_command.Add( sizer_command_sort, 0, wx.EXPAND, 5 )


        sizer_main.Add( sizer_command, 1, wx.EXPAND, 5 )

        self.staticline_action = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        sizer_main.Add( self.staticline_action, 0, wx.BOTTOM|wx.EXPAND|wx.TOP, 5 )

        self.label_current_command = wx.StaticText( self, wx.ID_ANY, u"Current command is:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_current_command.Wrap( -1 )

        sizer_main.Add( self.label_current_command, 0, wx.ALL, 5 )

        self.parameterGrid = pg.PropertyGrid(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.propgrid.PG_DEFAULT_STYLE|wx.propgrid.PG_SPLITTER_AUTO_CENTER)
        sizer_main.Add( self.parameterGrid, 1, wx.ALL|wx.EXPAND, 5 )

        sizer_dialog_buttons = wx.StdDialogButtonSizer()
        self.sizer_dialog_buttonsOK = wx.Button( self, wx.ID_OK )
        sizer_dialog_buttons.AddButton( self.sizer_dialog_buttonsOK )
        self.sizer_dialog_buttonsCancel = wx.Button( self, wx.ID_CANCEL )
        sizer_dialog_buttons.AddButton( self.sizer_dialog_buttonsCancel )
        sizer_dialog_buttons.Realize();

        sizer_main.Add( sizer_dialog_buttons, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizer_main )
        self.Layout()
        sizer_main.Fit( self )

        self.Centre( wx.BOTH )

        # Connect Events
        self.Bind( wx.EVT_INIT_DIALOG, self.on_InitDialog )
        self.choice_target.Bind( wx.EVT_CHOICE, self.on_choice_target )
        self.choice_saved_lists.Bind( wx.EVT_CHOICE, self.on_choice_saved_lists )
        self.treeCtrl_commands_available.Bind( wx.EVT_TREE_ITEM_ACTIVATED, self.on_TreeCommandActivated )
        self.treeCtrl_commands_available.Bind( wx.EVT_TREE_SEL_CHANGED, self.on_TreeCommandSelChanged )
        self.button_add.Bind( wx.EVT_BUTTON, self.on_button_add )
        self.button_add.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_add )
        self.button_remove.Bind( wx.EVT_BUTTON, self.on_button_remove )
        self.button_remove.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_remove )
        self.button_open.Bind( wx.EVT_BUTTON, self.on_button_open )
        self.button_save.Bind( wx.EVT_BUTTON, self.on_button_save )
        self.button_save.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_save )
        self.listCtrl_commands.Bind( wx.EVT_LIST_ITEM_ACTIVATED, self.on_listCtrl_commands_Activated )
        self.listCtrl_commands.Bind( wx.EVT_LIST_ITEM_SELECTED, self.on_listCtrl_commands_Selected )
        self.listCtrl_commands.Bind( wx.EVT_UPDATE_UI, self.onUpdate_listCtrl_commands )
        self.button_up.Bind( wx.EVT_BUTTON, self.on_button_up )
        self.button_up.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_up )
        self.button_down.Bind( wx.EVT_BUTTON, self.on_button_down )
        self.button_down.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_down )
        self.button_clear.Bind( wx.EVT_BUTTON, self.on_button_clear )
        self.button_clear.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_clear )
        self.parameterGrid.Bind( pg.EVT_PG_CHANGED, self.on_propertyGrid_Changed )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def on_InitDialog( self, event ):
        event.Skip()

    def on_choice_target( self, event ):
        event.Skip()

    def on_choice_saved_lists( self, event ):
        event.Skip()

    def on_TreeCommandActivated( self, event ):
        event.Skip()

    def on_TreeCommandSelChanged( self, event ):
        event.Skip()

    def on_button_add( self, event ):
        event.Skip()

    def onUpdate_button_add( self, event ):
        event.Skip()

    def on_button_remove( self, event ):
        event.Skip()

    def onUpdate_button_remove( self, event ):
        event.Skip()

    def on_button_open( self, event ):
        event.Skip()

    def on_button_save( self, event ):
        event.Skip()

    def onUpdate_button_save( self, event ):
        event.Skip()

    def on_listCtrl_commands_Activated( self, event ):
        event.Skip()

    def on_listCtrl_commands_Selected( self, event ):
        event.Skip()

    def onUpdate_listCtrl_commands( self, event ):
        event.Skip()

    def on_button_up( self, event ):
        event.Skip()

    def onUpdate_button_up( self, event ):
        event.Skip()

    def on_button_down( self, event ):
        event.Skip()

    def onUpdate_button_down( self, event ):
        event.Skip()

    def on_button_clear( self, event ):
        event.Skip()

    def onUpdate_button_clear( self, event ):
        event.Skip()

    def on_propertyGrid_Changed( self, event ):
        event.Skip()





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

from .hintPSpanel import StemPanel
from .hintPSpanel import ZonePanel
import wx
import wx.xrc

###########################################################################
## Class FontInfoHintingPSUI
###########################################################################

class FontInfoHintingPSUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 416,486 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )

        sizer = wx.BoxSizer( wx.VERTICAL )

        boxMain = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Main" ), wx.VERTICAL )

        sizerMain = wx.GridBagSizer( 0, 0 )
        sizerMain.SetFlexibleDirection( wx.HORIZONTAL )
        sizerMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_postscriptBlueFuzz = wx.StaticText( self, wx.ID_ANY, u"Blue Fuzz", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptBlueFuzz.Wrap( -1 )

        self.lbl_postscriptBlueFuzz.SetMinSize( wx.Size( 170,-1 ) )

        sizerMain.Add( self.lbl_postscriptBlueFuzz, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptBlueFuzz = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptBlueFuzz" )
        sizerMain.Add( self.txtCtrl_postscriptBlueFuzz, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_postscriptBlueShift = wx.StaticText( self, wx.ID_ANY, u"Blue Shift", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptBlueShift.Wrap( -1 )

        sizerMain.Add( self.lbl_postscriptBlueShift, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptBlueShift = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptBlueShift" )
        sizerMain.Add( self.txtCtrl_postscriptBlueShift, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_postscriptBlueScale = wx.StaticText( self, wx.ID_ANY, u"Blue Scale", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptBlueScale.Wrap( -1 )

        sizerMain.Add( self.lbl_postscriptBlueScale, wx.GBPosition( 2, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptBlueScale = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptBlueScale" )
        sizerMain.Add( self.txtCtrl_postscriptBlueScale, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_postscriptForceBold = wx.StaticText( self, wx.ID_ANY, u"Force Bold", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptForceBold.Wrap( -1 )

        sizerMain.Add( self.lbl_postscriptForceBold, wx.GBPosition( 3, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.checkBox_postscriptForceBold = wx.CheckBox( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptForceBold" )
        sizerMain.Add( self.checkBox_postscriptForceBold, wx.GBPosition( 3, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizerMain.AddGrowableCol( 1 )

        boxMain.Add( sizerMain, 1, wx.EXPAND, 5 )


        sizer.Add( boxMain, 0, wx.ALL|wx.EXPAND, 5 )

        boxStems = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Stems" ), wx.VERTICAL )

        sizerStems = wx.GridBagSizer( 0, 0 )
        sizerStems.SetFlexibleDirection( wx.BOTH )
        sizerStems.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )


        sizerStems.Add( ( 170, 0 ), wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_postscriptStemSnapH = wx.StaticText( self, wx.ID_ANY, u"Horizontal Stems", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptStemSnapH.Wrap( -1 )

        self.lbl_postscriptStemSnapH.SetMinSize( wx.Size( 170,-1 ) )

        sizerStems.Add( self.lbl_postscriptStemSnapH, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_postscriptStemSnapV = wx.StaticText( self, wx.ID_ANY, u"Vertical Stems", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptStemSnapV.Wrap( -1 )

        sizerStems.Add( self.lbl_postscriptStemSnapV, wx.GBPosition( 0, 2 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.m_button3 = wx.Button( self, wx.ID_ANY, u"MyButton", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_button3.SetMinSize( wx.Size( 160,40 ) )

        sizerStems.Add( self.m_button3, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.panel_postscriptStemSnapH = StemPanel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL, u"postscriptStemSnapH" )
        sizerStems.Add( self.panel_postscriptStemSnapH, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.panel_postscriptStemSnapV = StemPanel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL, u"postscriptStemSnapV" )
        sizerStems.Add( self.panel_postscriptStemSnapV, wx.GBPosition( 1, 2 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )


        boxStems.Add( sizerStems, 1, wx.EXPAND, 5 )


        sizer.Add( boxStems, 0, wx.ALL|wx.EXPAND, 5 )

        boxZones = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Allignment Zones" ), wx.VERTICAL )

        gridZones = wx.GridBagSizer( 0, 0 )
        gridZones.SetFlexibleDirection( wx.BOTH )
        gridZones.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_localZones = wx.StaticText( self, wx.ID_ANY, u"Local Zones", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_localZones.Wrap( -1 )

        self.lbl_localZones.SetMinSize( wx.Size( 170,-1 ) )

        gridZones.Add( self.lbl_localZones, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_postscriptBlueValues = wx.StaticText( self, wx.ID_ANY, u"Blue Values", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptBlueValues.Wrap( -1 )

        self.lbl_postscriptBlueValues.SetMinSize( wx.Size( 170,-1 ) )

        gridZones.Add( self.lbl_postscriptBlueValues, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_postscriptOtherBlues = wx.StaticText( self, wx.ID_ANY, u"Other Blues", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptOtherBlues.Wrap( -1 )

        gridZones.Add( self.lbl_postscriptOtherBlues, wx.GBPosition( 0, 2 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.button_copy_localZones = wx.Button( self, wx.ID_ANY, u"Copy local zones\nto family zones", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.button_copy_localZones.SetMinSize( wx.Size( 160,40 ) )

        gridZones.Add( self.button_copy_localZones, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.panel_postscriptBlueValues = ZonePanel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL, u"postscriptBlueValues" )
        gridZones.Add( self.panel_postscriptBlueValues, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.panel_postscriptOtherBlues = ZonePanel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL, u"postscriptOtherBlues" )
        gridZones.Add( self.panel_postscriptOtherBlues, wx.GBPosition( 1, 2 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_FamilyZones = wx.StaticText( self, wx.ID_ANY, u"Family Zones", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_FamilyZones.Wrap( -1 )

        gridZones.Add( self.lbl_FamilyZones, wx.GBPosition( 2, 0 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_postscriptFamilyBlues = wx.StaticText( self, wx.ID_ANY, u"Family Blues", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptFamilyBlues.Wrap( -1 )

        self.lbl_postscriptFamilyBlues.SetMinSize( wx.Size( 170,-1 ) )

        gridZones.Add( self.lbl_postscriptFamilyBlues, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_postscriptFamilyOtherBlues = wx.StaticText( self, wx.ID_ANY, u"Family Other Blues", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptFamilyOtherBlues.Wrap( -1 )

        gridZones.Add( self.lbl_postscriptFamilyOtherBlues, wx.GBPosition( 2, 2 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.button_copy_familyZones = wx.Button( self, wx.ID_ANY, u"Copy family zones\nto other fonts", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.button_copy_familyZones.SetMinSize( wx.Size( 160,40 ) )

        gridZones.Add( self.button_copy_familyZones, wx.GBPosition( 3, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.panel_postscriptFamilyBlues = ZonePanel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL, u"postscriptFamilyBlues" )
        gridZones.Add( self.panel_postscriptFamilyBlues, wx.GBPosition( 3, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.panel_postscriptFamilyOtherBlues = ZonePanel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL, u"postscriptFamilyOtherBlues" )
        gridZones.Add( self.panel_postscriptFamilyOtherBlues, wx.GBPosition( 3, 2 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )


        boxZones.Add( gridZones, 1, wx.EXPAND, 5 )


        sizer.Add( boxZones, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizer )
        self.Layout()

        # Connect Events
        self.button_copy_localZones.Bind( wx.EVT_BUTTON, self.on_button_copy_localZones )
        self.button_copy_familyZones.Bind( wx.EVT_BUTTON, self.on_button_copy_familyZones )
        self.button_copy_familyZones.Bind( wx.EVT_UPDATE_UI, self.onUpdate_button_copy_familyZones )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def on_button_copy_localZones( self, event ):
        event.Skip()

    def on_button_copy_familyZones( self, event ):
        event.Skip()

    def onUpdate_button_copy_familyZones( self, event ):
        event.Skip()





import logging
from string import digits

import wx

from .zonePanelUI import ZonePanelUI
from .stemPanelUI import StemPanelUI

log = logging.getLogger(__name__)

class NumberValidator(wx.Validator):
    def __init__(self):
        super().__init__()
        self._validChars = digits + ".+-"
        self.Bind(wx.EVT_CHAR, self.OnChar)

    def Clone(self):
        return NumberValidator()

    def Validate(self, win):
        ctrl = self.GetWindow()
        if not ctrl.Enabled:
            return True
        value = ctrl.GetValue()
        for c in value:
            if c not in self._validChars:
                return False
        return True

    def OnChar(self, event):
        key = event.GetKeyCode()
        log.debug("key: %r, chr: %r", key, chr(key))

        if key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255:
            event.Skip()
            return

        if chr(key) in self._validChars:
            event.Skip()
            return

        if not wx.Validator.IsSilent():
            wx.Bell()
        return

    def TransferToWindow(self):
        return True

    def TransferFromWindow(self):
        return True


class ZonePanel(ZonePanelUI):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.BORDER_NONE | wx.TAB_TRAVERSAL,
        name=wx.EmptyString,
    ):
        ZonePanelUI.__init__(
            self, parent, id=id, pos=pos, size=size, style=style, name=name
        )
        self._value = []
        self._maxValueLen = 0
        self._newZone = False
        self.textCtrl_bottom.Validator = NumberValidator()
        self.textCtrl_top.Validator = NumberValidator()

    def GetValue(self):
        return self._value

    def SetValue(self, value):
        if value == self._value:
            return
        if isinstance(value, (list, tuple)) and len(value) % 2 == 0:
            self._value = value
            self.listCtrl_zone.SetItemCount(int(len(value) / 2))
            self.listCtrl_zone.Refresh()
            self.Validator.TransferFromWindow()

    Value = property(GetValue, SetValue)

    def on_listCtrl_zone_selected(self, event):
        self.textCtrl_bottom.Value = str(self.Value[event.Index * 2])
        self.textCtrl_top.Value = str(self.Value[event.Index * 2 + 1])
        self._newZone = False

    def onUpdate_editControls(self, event):
        do_edit = self._newZone or self.listCtrl_zone.SelectedItemCount > 0
        if do_edit:
            event.Enable(True)
        else:
            self.textCtrl_bottom.Value = ""
            self.textCtrl_top.Value = ""
            event.Enable(False)

    def on_button_apply(self, event):
        try:
            bottom = int(self.textCtrl_bottom.Value)
        except ValueError:
            try:
                bottom = float(self.textCtrl_bottom.Value)
            except ValueError:
                wx.LogWarning(
                    f"Invalid value for bottom edge of zone\n\nCan't convert '{self.textCtrl_bottom.Value}' to number."
                )
                self.textCtrl_bottom.SelectAll()
                self.textCtrl_bottom.SetFocus()
                return
        try:
            top = int(self.textCtrl_top.Value)
        except ValueError:
            try:
                top = float(self.textCtrl_top.Value)
            except ValueError:
                wx.LogWarning(
                    f"Invalid value for top edge of zone\n\nCan't convert '{self.textCtrl_top.Value}' to number."
                )
                self.textCtrl_top.SelectAll()
                self.textCtrl_top.SetFocus()
                return
        zones = self.Value[:]
        if not self._newZone:
            idx = self.listCtrl_zone.FocusedItem
            del zones[idx * 2 : idx * 2 + 2]
        zones.extend([bottom, top])
        zones.sort()
        self.Value = zones
        # self.Validator.TransferFromWindow()
        self._newZone = False
        # event.Skip()

    def on_button_add(self, event):
        self._newZone = True
        self.textCtrl_bottom.Value = "0"
        self.textCtrl_top.Value = "0"

    def onUpdate_button_add(self, event):
        event.Enable(len(self._value) < self._maxValueLen)

    def on_button_remove(self, event):
        zones = self.Value[:]
        idx = self.listCtrl_zone.FocusedItem
        del zones[idx * 2 : idx * 2 + 2]
        self.Value = zones

    def onUpdate_button_remove(self, event):
        event.Enable(self.listCtrl_zone.SelectedItemCount > 0)


class StemPanel(StemPanelUI):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.BORDER_NONE | wx.TAB_TRAVERSAL,
        name=wx.EmptyString,
    ):
        StemPanelUI.__init__(
            self, parent, id=id, pos=pos, size=size, style=style, name=name
        )
        self._value = []
        self._maxValueLen = 12
        self._newStem = False
        self.textCtrl_width.Validator = NumberValidator()

    def GetValue(self):
        return self._value

    def SetValue(self, value):
        if value == self._value:
            return
        if isinstance(value, (list, tuple)):
            self._value = value
            self.listCtrl_stem.SetItemCount(len(value))
            self.listCtrl_stem.Refresh()
            self.Validator.TransferFromWindow()

    Value = property(GetValue, SetValue)

    def on_listCtrl_stem_selected(self, event):
        self.textCtrl_width.Value = str(self.Value[event.Index])
        self._newStem = False

    def onUpdate_editControls(self, event):
        do_edit = self._newStem or self.listCtrl_stem.SelectedItemCount > 0
        if do_edit:
            event.Enable(True)
        else:
            self.textCtrl_width.Value = ""
            event.Enable(False)

    def on_button_apply(self, event):
        try:
            width = int(self.textCtrl_width.Value)
        except ValueError:
            try:
                width = float(self.textCtrl_width.Value)
            except ValueError:
                wx.LogWarning(
                    f"Invalid value for width of stem\n\nCan't convert '{self.textCtrl_width.Value}' to number."
                )
                self.textCtrl_width.SelectAll()
                self.textCtrl_width.SetFocus()
                return
        stems = self.Value[:]
        if not self._newStem:
            idx = self.listCtrl_stem.FocusedItem
            del stems[idx]
        stems.append(width)
        stems.sort()
        self.Value = stems
        self._newStem = False

    def on_button_add(self, event):
        self._newStem = True
        self.textCtrl_width.Value = "0"

    def onUpdate_button_add(self, event):
        event.Enable(len(self._value) < self._maxValueLen)

    def on_button_remove(self, event):
        stems = self.Value[:]
        idx = self.listCtrl_stem.FocusedItem
        del stems[idx]
        self.Value = stems

    def onUpdate_button_remove(self, event):
        event.Enable(self.listCtrl_stem.SelectedItemCount > 0)




from __future__ import annotations

from typing import TYPE_CHECKING, Optional, Tuple

import wbFontParts
import wx
from fontParts import world
from wbBase.document import DOC_NEW, DOC_NO_VIEW, DOC_SILENT
from wbBase.panelManager import toolbarPaneInfo
from wbFontParts import (
    RAnchor,
    RComponent,
    RContour,
    RFeatures,
    RGlyph,
    RGroups,
    RGuideline,
    RImage,
    RInfo,
    RKerning,
    RLayer,
    RLib,
    RPoint,
    RSegment,
    mark,
)

from .dialog.findGlyphDialog import FindGlyphDialog
from .document import UfoDocument
from .panel.glyphViewPanel import GlyphViewPanel, glyphViewPanelInfo
from .template import GlyphsTemplate, UfoTemplate, VFBTemplate
from .toolbar.command import GlyphCommandToolbar
from .toolbar.editTool import GlyphEditToolbar
from .toolbar.glyphLock import GlyphLockToolbar
from .toolbar.glyphShow import GlyphShowToolbar
from .toolbar.layer import LayerToolbar
from .toolbar.markColor import GlyphMarkToolbar
from .view.font import UfoFontView
from .view.glyph import UfoGlyphView

if TYPE_CHECKING:
    from wbBase.application import App

__version__ = "0.2.1"


def getApp() -> App:
    """
    The currently running Workbench application.
    """
    return wx.GetApp()


# add wbFontParts objects to the world
world.dispatcher["RAnchor"] = RAnchor
world.dispatcher["RComponent"] = RComponent
world.dispatcher["RContour"] = RContour
world.dispatcher["RFeatures"] = RFeatures
world.dispatcher["RGlyph"] = RGlyph
world.dispatcher["RGroups"] = RGroups
world.dispatcher["RGuideline"] = RGuideline
world.dispatcher["RImage"] = RImage
world.dispatcher["RInfo"] = RInfo
world.dispatcher["RKerning"] = RKerning
world.dispatcher["RLayer"] = RLayer
world.dispatcher["RLib"] = RLib
world.dispatcher["RPoint"] = RPoint


def dispatchUItools(app: App) -> None:
    uitools = app.pluginManager.get("uitools")
    if uitools:
        for attr in ("AskString", "AskYesNoCancel", "GetFile", "Message", "PutFile"):
            if hasattr(uitools, attr):
                world.dispatcher[attr] = getattr(uitools, attr)


def CurrentFont() -> Optional[wbFontParts.RFont]:
    doc = getApp().documentManager.currentDocument
    if isinstance(doc, UfoDocument):
        return doc.RFont


world.dispatcher["CurrentFont"] = CurrentFont


def CurrentGlyph() -> Optional[RGlyph]:
    view = getApp().documentManager.currentView
    if view:
        if view.typeName == "UFO Font View":
            glyph = view.frame.glyphGridPanel.currentGlyph
            if glyph is not None:
                return view.document.RFont[glyph.name]
        elif view.typeName == "UFO Glyph View":
            return view.document.RFont.getLayer(view.frame.canvas.glyph.layer.name)[
                view.glyph.name
            ]


world.dispatcher["CurrentGlyph"] = CurrentGlyph


def CurrentAnchors() -> Tuple[RAnchor, ...]:
    """
    :return: Selected anchors in the current glyph.
    """
    glyph = CurrentGlyph()
    if glyph is None:
        return ()
    return glyph.selectedAnchors


world.dispatcher["CurrentAnchors"] = CurrentAnchors


def CurrentComponents() -> Tuple[RComponent, ...]:
    """
    :return: Selected components in the current glyph.
    """
    glyph = CurrentGlyph()
    if glyph is None:
        return ()
    return glyph.selectedComponents


world.dispatcher["CurrentComponents"] = CurrentComponents


def CurrentContours() -> Tuple[RContour, ...]:
    """
    :return: Selected contours in the current glyph.
    """
    glyph = CurrentGlyph()
    if glyph is None:
        return ()
    return glyph.selectedContours


world.dispatcher["CurrentContours"] = CurrentContours


def CurrentPoints() -> Tuple[RPoint, ...]:
    """
    :return: Selected points in the current glyph.
    """
    glyph = CurrentGlyph()
    if glyph is None:
        return ()
    points = []
    for contour in glyph:
        points.extend(contour.selectedPoints)
    return tuple(points)


world.dispatcher["CurrentPoints"] = CurrentPoints


def CurrentSegments() -> Tuple[RSegment, ...]:
    """
    :return: Selected segments in the current glyph.
    """
    glyph = CurrentGlyph()
    if glyph is None:
        return ()
    segments = []
    for contour in glyph:
        segments.extend(contour.selectedSegments)
    return tuple(segments)


world.dispatcher["CurrentSegments"] = CurrentSegments


def CurrentGuidelines() -> Tuple[RGuideline, ...]:
    guidelines = []
    font = CurrentFont()
    if font is not None:
        guidelines.extend(font.selectedGuidelines)
    glyph = CurrentGlyph()
    if glyph is not None:
        guidelines.extend(glyph.selectedGuidelines)
    return tuple(guidelines)


world.dispatcher["CurrentGuidelines"] = CurrentGuidelines


def CurrentLayer() -> Optional[RLayer]:
    view = getApp().documentManager.currentView
    if isinstance(view, UfoFontView):
        return view.document.RFont.defaultLayer
    elif isinstance(view, UfoGlyphView):
        return view.document.RFont.getLayer(view.frame.canvas.glyph.layer.name)


world.dispatcher["CurrentLayer"] = CurrentLayer


def AllFonts(sortOptions=None):
    fontlist = [
        doc.RFont
        for doc in getApp().documentManager.getDocumentsByTypeName("UFO document")
    ]
    fontlist = world.FontList(fontlist)
    if sortOptions is not None:
        fontlist.sortBy(sortOptions)
    return fontlist


world.dispatcher["AllFonts"] = AllFonts


def NewFont(familyName=None, styleName=None, showInterface=True) -> wbFontParts.RFont:
    app = getApp()
    ufoTemplate = None
    for template in app.documentManager.templates:
        if isinstance(template, UfoTemplate):
            ufoTemplate = template
            break
    if ufoTemplate:
        if showInterface:
            flags = DOC_NEW
        else:
            flags = DOC_NEW | DOC_NO_VIEW
        newDoc = ufoTemplate.CreateDocument("", flags)
        if isinstance(newDoc, UfoDocument):
            # newDoc._typeName = ufoTemplate.documentTypeName
            # newDoc.template = ufoTemplate
            newDoc.OnNewDocument()
            font = newDoc.RFont
            title = []
            if familyName is not None:
                font.info.familyName = familyName
                title.append(str(familyName).strip())
            if styleName is not None:
                font.info.styleName = styleName
                title.append(str(styleName).strip())
            if title:
                newDoc.title = "-".join(title)
            return font


world.dispatcher["NewFont"] = NewFont


def OpenFont(path:str, showInterface:bool=True) -> Optional[wbFontParts.RFont:]:
    app = getApp()
    documentManager = app.TopWindow.documentManager
    template = documentManager.FindTemplateForPath(path)
    if isinstance(template, UfoTemplate):
        if showInterface:
            flags = DOC_SILENT
        else:
            flags = DOC_SILENT | DOC_NO_VIEW
        newDoc = documentManager.CreateDocument(path, flags)
        if isinstance(newDoc, UfoDocument):
            return newDoc.RFont


world.dispatcher["OpenFont"] = OpenFont


def RFont(pathOrObject=None, showInterface=True) -> Optional[wbFontParts.RFont:]:
    if pathOrObject is None:
        return NewFont(showInterface=showInterface)
    return OpenFont(pathOrObject, showInterface=showInterface)


world.dispatcher["RFont"] = RFont

# ============================================================================
# fontParts.ui
# ============================================================================


def FindGlyph(aFont, message="Search for a glyph:", title="UFO Workbench"):
    glyphName = None
    with FindGlyphDialog(wx.GetApp().TopWindow, aFont) as dialog:
        if dialog.ShowModal() == wx.ID_OK:
            glyphName = dialog.selctedGlyph
    if glyphName:
        return aFont[glyphName]


def SelectFonts(message="Select fonts:", title="UFO Workbench", allFonts=None):
    result = []
    if allFonts is None:
        allFonts = AllFonts()
    if allFonts:
        from .dialog.selectFontsDialog import SelectFontsDialog

        with SelectFontsDialog(message, title, allFonts) as dialog:
            if dialog.ShowModal() == wx.ID_OK:
                result = dialog.selectedFonts
    else:
        wx.LogWarning("Can not select fonts!\n\nNo fonts open.")
    return result


# ============================================================================
# additional stuff
# ============================================================================


def SelectSingleFont(message="Select font:", title="UFO Workbench", allFonts=None):
    result = None
    if allFonts is None:
        allFonts = AllFonts()
    if allFonts:
        from .dialog.selectFontsDialog import SelectFontsDialog

        with SelectFontsDialog(message, title, allFonts) as dialog:
            dialog.listCtrl_fonts.WindowStyle |= wx.LC_SINGLE_SEL
            if dialog.ShowModal() == wx.ID_OK:
                if dialog.selectedFonts:
                    result = dialog.selectedFonts[0]
    else:
        wx.LogWarning("Can not select font!\n\nNo fonts open.")
    return result


def ShowFont(font) -> Optional[UfoDocument]:
    app = getApp()
    ufoTemplate = None
    for template in app.documentManager.templates:
        if isinstance(template, UfoTemplate):
            ufoTemplate = template
            break
    if isinstance(ufoTemplate, UfoTemplate):
        doc = ufoTemplate.CreateDocument("", DOC_NEW)
        if isinstance(doc, UfoDocument):
            doc.font = font
            doc.beginObservation()
            doc.UpdateAllViews(doc, ("font loaded", True))
            return doc
    return None


doctemplates = [UfoTemplate, VFBTemplate, GlyphsTemplate]

toolbars = [
    (GlyphCommandToolbar, toolbarPaneInfo("Glyph Commands")),
    (GlyphEditToolbar, toolbarPaneInfo("Glyph Tools")),
    (GlyphLockToolbar, toolbarPaneInfo("Glyph Lock")),
    (GlyphMarkToolbar, toolbarPaneInfo("Glyph MarkColor")),
    (GlyphShowToolbar, toolbarPaneInfo("Glyph Show")),
    (LayerToolbar, toolbarPaneInfo("Layer")),
]

panels = [(GlyphViewPanel, glyphViewPanelInfo)]

globalObjects = [
    "AllFonts",
    "AskString",
    "AskYesNoCancel",
    "CurrentAnchors",
    "CurrentComponents",
    "CurrentContours",
    "CurrentFont",
    "CurrentGlyph",
    "CurrentGuidelines",
    "CurrentLayer",
    "CurrentPoints",
    "CurrentSegments",
    "FindGlyph",
    "getCheckList",
    "GetFile",
    "mark",
    "Message",
    "NewFont",
    "OpenFont",
    "PutFile",
    "RFont",
    "RGlyph",
    "SelectFonts",
    "SelectSingleFont",
    "ShowFont",
]

getApp().AddPostInitAction(dispatchUItools)




import logging

import wx

from .fontInfoHintingPSUI import FontInfoHintingPSUI

log = logging.getLogger(__name__)

class FontInfoHintingPS(FontInfoHintingPSUI):
    """
    Font Info page for PostScript hinting
    """
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.BORDER_NONE | wx.TAB_TRAVERSAL,
    ):
        super().__init__(parent, id, pos, size, style, name="FontInfoHintingPS")
        self.SetScrollRate(5, 5)
        self.panel_postscriptBlueValues._maxValueLen = 14
        self.panel_postscriptOtherBlues._maxValueLen = 10
        self.panel_postscriptFamilyBlues._maxValueLen = 14
        self.panel_postscriptFamilyOtherBlues._maxValueLen = 10

    def __repr__(self):
        return f"<{self.__class__.__name__} of {self.Parent}>"

    def on_button_copy_localZones(self, event):
        self.panel_postscriptFamilyBlues.Value = self.panel_postscriptBlueValues.Value
        self.panel_postscriptFamilyOtherBlues.Value = self.panel_postscriptOtherBlues.Value

    def on_button_copy_familyZones(self, event):
        from ... import AllFonts, SelectFonts
        fonts = [f for f in AllFonts() if f != self.Parent.font]
        selectedFonts = SelectFonts("Select fonts to copy family zones to", "Copy Family Zones", fonts)
        if selectedFonts:
            for font in selectedFonts:
                info = font.info
                info.postscriptFamilyBlues = self.panel_postscriptFamilyBlues.Value
                info.postscriptFamilyOtherBlues = self.panel_postscriptFamilyOtherBlues.Value

    def onUpdate_button_copy_familyZones(self, event):
        from ... import AllFonts
        event.Enable(len(AllFonts()) > 1)



import os
import wx
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin


class FontSelectListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.LC_REPORT | wx.LC_VIRTUAL,
        validator=wx.DefaultValidator,
        name="FontSelectListCtrl",
    ):
        wx.ListCtrl.__init__(self, parent, id, pos, size, style, validator, name)
        ListCtrlAutoWidthMixin.__init__(self)
        self.InsertColumn(0, "Font")
        self.InsertColumn(1, "Path")
        self.SetColumnWidth(0, 200)
        self.allFonts = self.Parent.allFonts
        self.SetItemCount(len(self.allFonts))

    @property
    def app(self):
        return wx.GetApp()

    def getSelectedFonts(self):
        fonts = []
        for i in range(self.ItemCount):
            if self.GetItemState(i, wx.LIST_STATE_SELECTED):
                fonts.append(self.allFonts[i])
        return fonts

    def OnGetItemText(self, item, col):
        font = self.allFonts[item]
        if col == 0:
            return f"{font.info.familyName} {font.info.styleName}"
        if col == 1:
            if font.path:
                return font.path
            return "Not saved yet"




"""
commandListCtrl
===============================================================================
"""

import wx
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin


class CommandListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.LC_NO_HEADER | wx.LC_REPORT | wx.LC_SINGLE_SEL | wx.LC_VIRTUAL,
        validator=wx.DefaultValidator,
        name="CommandListCtrl",
    ):
        wx.ListCtrl.__init__(self, parent, id, pos, size, style, validator, name)
        self.InsertColumn(0, "Command")
        ListCtrlAutoWidthMixin.__init__(self)
        self.setResizeColumn(0)
        self.SetItemCount(0)

    def OnGetItemText(self, item, col):
        return str(self.Parent.commandList[item])

    # def getColumnText(self, index, col):
    # 	item = self.GetItem(index, col)
    # 	return item.GetText()



import logging

import wx
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin, TextEditMixin

log = logging.getLogger(__name__)


class BitList(wx.CheckListBox):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        choices=None,
        style=wx.LB_EXTENDED | wx.LB_NEEDED_SB,
        validator=wx.DefaultValidator,
        name="BitList",
    ):
        if not choices:
            choices = []
        super().__init__(parent, id, pos, size, choices, style, validator, name)

    def GetValue(self):
        return list(self.GetCheckedItems())

    def SetValue(self, bitList):
        self.SetCheckedItems(bitList)

    Value = property(GetValue, SetValue)


class CodePageBitList(BitList):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        choices=None,
        style=wx.LB_EXTENDED | wx.LB_MULTIPLE | wx.LB_NEEDED_SB,
        validator=wx.DefaultValidator,
        name="openTypeOS2CodePageRanges",
    ):
        choices = [
            "Latin 1 (1252)",
            "Latin 2 Eastern Europe (1250)",
            "Cyrillic (1251)",
            "Greek (1253)",
            "Turkish (1254)",
            "Hebrew (1255)",
            "Arabic (1256)",
            "Windows Baltic (1257)",
            "Vietnamese (1258)",
            "Reserved for Alternate ANSI",
            "Reserved for Alternate ANSI",
            "Reserved for Alternate ANSI",
            "Reserved for Alternate ANSI",
            "Reserved for Alternate ANSI",
            "Reserved for Alternate ANSI",
            "Reserved for Alternate ANSI",
            "Thai (874)",
            "JIS/Japan (932)",
            "Chinese Simplified PRC and Singapore (936)",
            "Korean Wansung (949)",
            "Chinese Traditional Taiwan and Hong Kong (950)",
            "Korean Johab (1361)",
            "Reserved for Alternate ANSI & OEM",
            "Reserved for Alternate ANSI & OEM",
            "Reserved for Alternate ANSI & OEM",
            "Reserved for Alternate ANSI & OEM",
            "Reserved for Alternate ANSI & OEM",
            "Reserved for Alternate ANSI & OEM",
            "Reserved for Alternate ANSI & OEM",
            "Macintosh Character Set (US Roman)",
            "OEM Character Set",
            "Symbol Character Set",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "Reserved for OEM",
            "IBM Greek (869)",
            "MS-DOS Russian ()",
            "MS-DOS Nordic (866)",
            "Arabic (864)",
            "MS-DOS Canadian French (863)",
            "Hebrew (862)",
            "MS-DOS Icelandic (861)",
            "MS-DOS Portuguese (860)",
            "IBM Turkish (857)",
            "IBM Cyrillic; primarily Russian (855)",
            "Latin 2 (852)",
            "MS-DOS Baltic (775)",
            "Greek; former 437 G (737)",
            "Arabic; ASMO 708 (708)",
            "WE/Latin 1 (850)",
            "US (437)",
        ]

        super().__init__(parent, id, pos, size, choices, style, validator, name)


class UnicodeRangeBitList(BitList):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        choices=None,
        style=wx.LB_EXTENDED | wx.LB_MULTIPLE | wx.LB_NEEDED_SB,
        validator=wx.DefaultValidator,
        name="openTypeOS2UnicodeRanges",
    ):
        choices = [
            "Basic Latin",
            "Latin-1 Supplement",
            "Latin Extended-A",
            "Latin Extended-B",
            "IPA Extensions",
            "Spacing Modifier Letters",
            "Combining Diacritical Marks",
            "Greek and Coptic",
            "Coptic",
            "Cyrillic",
            "Armenian",
            "Hebrew",
            "Vai",
            "Arabic",
            "NKo",
            "Devanagari",
            "Bengali",
            "Gurmukhi",
            "Gujarati",
            "Oriya",
            "Tamil",
            "Telugu",
            "Kannada",
            "Malayalam",
            "Thai",
            "Lao",
            "Georgian",
            "Balinese",
            "Hangul Jamo",
            "Latin Extended Additional",
            "Greek Extended",
            "General Punctuation",
            "Superscripts And Subscripts",
            "Currency Symbols",
            "Combining Diacritical Marks For Symbols",
            "Letterlike Symbols",
            "Number Forms",
            "Arrows",
            "Mathematical Operators",
            "Miscellaneous Technical",
            "Control Pictures",
            "Optical Character Recognition",
            "Enclosed Alphanumerics",
            "Box Drawing",
            "Block Elements",
            "Geometric Shapes",
            "Miscellaneous Symbols",
            "Dingbats",
            "CJK Symbols And Punctuation",
            "Hiragana",
            "Katakana",
            "Bopomofo",
            "Hangul Compatibility Jamo",
            "Phags-pa",
            "Enclosed CJK Letters And Months",
            "CJK Compatibility",
            "Hangul Syllables",
            "Non-Plane 0 *",
            "Phoenician",
            "CJK Unified Ideographs",
            "Private Use Area (plane 0)",
            "CJK Strokes",
            "Alphabetic Presentation Forms",
            "Arabic Presentation Forms-A",
            "Combining Half Marks",
            "Vertical Forms",
            "Small Form Variants",
            "Arabic Presentation Forms-B",
            "Halfwidth And Fullwidth Forms",
            "Specials",
            "Tibetan",
            "Syriac",
            "Thaana",
            "Sinhala",
            "Myanmar",
            "Ethiopic",
            "Cherokee",
            "Unified Canadian Aboriginal Syllabics",
            "Ogham",
            "Runic",
            "Khmer",
            "Mongolian",
            "Braille Patterns",
            "Yi Syllables and Radicals",
            "Tagalog, Hanunoo, Buhid, Tagbanwa",
            "Old Italic",
            "Gothic",
            "Deseret",
            "Musical Symbols Byzantine and Ancient Greek",
            "Mathematical Alphanumeric Symbols",
            "Private Use (plane 15 and 16)",
            "Variation Selectors",
            "Tags",
            "Limbu",
            "Tai Le",
            "New Tai Lue",
            "Buginese",
            "Glagolitic",
            "Tifinagh",
            "Yijing Hexagram Symbols",
            "Syloti Nagri",
            "Linear B Syllabary and Ideograms",
            "Ancient Greek Numbers",
            "Ugaritic",
            "Old Persian",
            "Shavian",
            "Osmanya",
            "Cypriot Syllabary",
            "Kharoshthi",
            "Tai Xuan Jing Symbols",
            "Cuneiform",
            "Counting Rod Numerals",
            "Sundanese",
            "Lepcha",
            "Ol Chiki",
            "Saurashtra",
            "Kayah Li",
            "Rejang",
            "Cham",
            "Ancient Symbols",
            "Phaistos Disc",
            "Carian, Lycian, Lydian",
            "Domino and Mahjong Tiles",
            "Reserved for process-internal usage",
            "Reserved for process-internal usage",
            "Reserved for process-internal usage",
            "Reserved for process-internal usage",
            "Reserved for process-internal usage",
        ]

        super().__init__(parent, id, pos, size, choices, style, validator, name)

class BlueListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin, TextEditMixin):
    maxItems = 7
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.LC_REPORT | wx.LC_VIRTUAL,
        name="BlueListCtrl",
    ):
        wx.ListCtrl.__init__(self, parent, id, pos, size, style, name=name)
        self._font = None
        ListCtrlAutoWidthMixin.__init__(self)
        self.InsertColumn(0, "Top")
        self.InsertColumn(1, "Bottom")
        self.InsertColumn(2, "Width")
        self.SetItemCount(0)
        TextEditMixin.__init__(self)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoCaretUI
###########################################################################

class FontInfoCaretUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )

        sizerMain = wx.BoxSizer( wx.VERTICAL )

        boxHhea = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"hhea" ), wx.VERTICAL )

        sizerHhea = wx.GridBagSizer( 0, 0 )
        sizerHhea.SetFlexibleDirection( wx.HORIZONTAL )
        sizerHhea.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeHheaCaretSlopeRise = wx.StaticText( self, wx.ID_ANY, u"Caret Slope Rise", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeHheaCaretSlopeRise.Wrap( -1 )

        self.lbl_openTypeHheaCaretSlopeRise.SetMinSize( wx.Size( 170,-1 ) )

        sizerHhea.Add( self.lbl_openTypeHheaCaretSlopeRise, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeHheaCaretSlopeRise = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeHheaCaretSlopeRise" )
        sizerHhea.Add( self.txtCtrl_openTypeHheaCaretSlopeRise, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeHheaCaretSlopeRun = wx.StaticText( self, wx.ID_ANY, u"Caret Slope Run", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeHheaCaretSlopeRun.Wrap( -1 )

        sizerHhea.Add( self.lbl_openTypeHheaCaretSlopeRun, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeHheaCaretSlopeRun = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeHheaCaretSlopeRun" )
        sizerHhea.Add( self.txtCtrl_openTypeHheaCaretSlopeRun, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeHheaCaretOffset = wx.StaticText( self, wx.ID_ANY, u"Caret Offset", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeHheaCaretOffset.Wrap( -1 )

        sizerHhea.Add( self.lbl_openTypeHheaCaretOffset, wx.GBPosition( 2, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeHheaCaretOffset = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeHheaCaretOffset" )
        sizerHhea.Add( self.txtCtrl_openTypeHheaCaretOffset, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizerHhea.AddGrowableCol( 1 )

        boxHhea.Add( sizerHhea, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxHhea, 0, wx.ALL|wx.EXPAND, 5 )

        boxVhea = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"vhea" ), wx.VERTICAL )

        sizerVhea = wx.GridBagSizer( 0, 0 )
        sizerVhea.SetFlexibleDirection( wx.HORIZONTAL )
        sizerVhea.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeVheaCaretSlopeRise = wx.StaticText( self, wx.ID_ANY, u"Caret Slope Rise", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeVheaCaretSlopeRise.Wrap( -1 )

        self.lbl_openTypeVheaCaretSlopeRise.SetMinSize( wx.Size( 170,-1 ) )

        sizerVhea.Add( self.lbl_openTypeVheaCaretSlopeRise, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeVheaCaretSlopeRise = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeVheaCaretSlopeRise" )
        sizerVhea.Add( self.txtCtrl_openTypeVheaCaretSlopeRise, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeVheaCaretSlopeRun = wx.StaticText( self, wx.ID_ANY, u"Caret Slope Run", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeVheaCaretSlopeRun.Wrap( -1 )

        sizerVhea.Add( self.lbl_openTypeVheaCaretSlopeRun, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeVheaCaretSlopeRun = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeVheaCaretSlopeRun" )
        sizerVhea.Add( self.txtCtrl_openTypeVheaCaretSlopeRun, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_openTypeVheaCaretOffset = wx.StaticText( self, wx.ID_ANY, u"Caret Offset", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeVheaCaretOffset.Wrap( -1 )

        sizerVhea.Add( self.lbl_openTypeVheaCaretOffset, wx.GBPosition( 2, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeVheaCaretOffset = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeVheaCaretOffset" )
        sizerVhea.Add( self.txtCtrl_openTypeVheaCaretOffset, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizerVhea.AddGrowableCol( 1 )

        boxVhea.Add( sizerVhea, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxVhea, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizerMain )
        self.Layout()
        sizerMain.Fit( self )

    def __del__( self ):
        pass





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Oct 26 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

from ..control.findGlyphListCtrl import FindGlyphListCtrl
import wx
import wx.xrc

###########################################################################
## Class FindGlyphDialogUI
###########################################################################

class FindGlyphDialogUI ( wx.Dialog ):

	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Find Glyph", pos = wx.DefaultPosition, size = wx.Size( 400,400 ), style = wx.CAPTION|wx.RESIZE_BORDER )

		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

		sizerMain = wx.FlexGridSizer( 0, 1, 0, 0 )
		sizerMain.AddGrowableCol( 0 )
		sizerMain.AddGrowableRow( 1 )
		sizerMain.SetFlexibleDirection( wx.BOTH )
		sizerMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		sizerTop = wx.BoxSizer( wx.HORIZONTAL )

		choice_findAttrChoices = [ u"Name", u"Unicode" ]
		self.choice_findAttr = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_findAttrChoices, 0 )
		self.choice_findAttr.SetSelection( 0 )
		sizerTop.Add( self.choice_findAttr, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		choice_findCompareChoices = [ u"equals to", u"starts with", u"ends with", u"contains" ]
		self.choice_findCompare = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_findCompareChoices, 0 )
		self.choice_findCompare.SetSelection( 1 )
		sizerTop.Add( self.choice_findCompare, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.textCtrl_findValue = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_PROCESS_ENTER )
		sizerTop.Add( self.textCtrl_findValue, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizerMain.Add( sizerTop, 0, wx.EXPAND, 5 )

		sizerCenter = wx.BoxSizer( wx.HORIZONTAL )

		self.listCtrl_glyphs = FindGlyphListCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LC_NO_HEADER|wx.LC_REPORT|wx.LC_SINGLE_SEL|wx.LC_SORT_ASCENDING|wx.LC_VIRTUAL )
		sizerCenter.Add( self.listCtrl_glyphs, 1, wx.EXPAND|wx.LEFT|wx.RIGHT, 5 )

		self.bitmapGlyph = wx.StaticBitmap( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.Size( 72,72 ), 0 )
		sizerCenter.Add( self.bitmapGlyph, 0, wx.LEFT|wx.RIGHT, 5 )


		sizerMain.Add( sizerCenter, 1, wx.EXPAND, 5 )

		self.staticline = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		sizerMain.Add( self.staticline, 0, wx.BOTTOM|wx.EXPAND|wx.TOP, 5 )

		sizerBottom = wx.BoxSizer( wx.HORIZONTAL )

		self.checkBox_create = wx.CheckBox( self, wx.ID_ANY, u"Create unexisting", wx.DefaultPosition, wx.DefaultSize, 0 )
		sizerBottom.Add( self.checkBox_create, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.buttonSelect = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

		self.buttonSelect.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_TICK_MARK, wx.ART_BUTTON ) )
		self.buttonSelect.Enable( False )
		self.buttonSelect.SetToolTip( u"Select all glyphs in the list" )

		sizerBottom.Add( self.buttonSelect, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		sizerDialogButtons = wx.StdDialogButtonSizer()
		self.sizerDialogButtonsOK = wx.Button( self, wx.ID_OK )
		sizerDialogButtons.AddButton( self.sizerDialogButtonsOK )
		self.sizerDialogButtonsCancel = wx.Button( self, wx.ID_CANCEL )
		sizerDialogButtons.AddButton( self.sizerDialogButtonsCancel )
		sizerDialogButtons.Realize();

		sizerBottom.Add( sizerDialogButtons, 1, wx.ALIGN_CENTER_VERTICAL, 5 )


		sizerMain.Add( sizerBottom, 0, wx.EXPAND, 5 )


		self.SetSizer( sizerMain )
		self.Layout()

		self.Centre( wx.BOTH )

		# Connect Events
		self.choice_findAttr.Bind( wx.EVT_CHOICE, self.onfindAttr_CHOICE )
		self.choice_findCompare.Bind( wx.EVT_CHOICE, self.onfindCompare_CHOICE )
		self.textCtrl_findValue.Bind( wx.EVT_TEXT, self.on_TEXT )
		self.textCtrl_findValue.Bind( wx.EVT_TEXT_ENTER, self.on_TEXT_ENTER )
		self.listCtrl_glyphs.Bind( wx.EVT_LIST_ITEM_ACTIVATED, self.on_LIST_ITEM_ACTIVATED )
		self.listCtrl_glyphs.Bind( wx.EVT_LIST_ITEM_SELECTED, self.on_LIST_ITEM_SELECTED )
		self.buttonSelect.Bind( wx.EVT_BUTTON, self.on_buttonSelect )
		self.buttonSelect.Bind( wx.EVT_UPDATE_UI, self.update_buttonSelect )

	def __del__( self ):
		pass


	# Virtual event handlers, overide them in your derived class
	def onfindAttr_CHOICE( self, event ):
		event.Skip()

	def onfindCompare_CHOICE( self, event ):
		event.Skip()

	def on_TEXT( self, event ):
		event.Skip()

	def on_TEXT_ENTER( self, event ):
		event.Skip()

	def on_LIST_ITEM_ACTIVATED( self, event ):
		event.Skip()

	def on_LIST_ITEM_SELECTED( self, event ):
		event.Skip()

	def on_buttonSelect( self, event ):
		event.Skip()

	def update_buttonSelect( self, event ):
		event.Skip()





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class GlyphGridStatusPanelUI
###########################################################################

class GlyphGridStatusPanelUI ( wx.Panel ):

	def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = 0, name = u"GlyphGridStatusPanel" ):
		wx.Panel.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

		self.SetFont( wx.Font( 8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, wx.EmptyString ) )
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DLIGHT ) )

		sizer = wx.BoxSizer( wx.HORIZONTAL )

		self.button_fontinfo = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

		self.button_fontinfo.SetBitmap( wx.ArtProvider.GetBitmap( "VIEW_INFO", wx.ART_BUTTON ) )
		self.button_fontinfo.SetToolTip( u"Open Font Info" )

		sizer.Add( self.button_fontinfo, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizer.Add( ( 0, 0), 1, wx.EXPAND, 0 )

		self.text_glyph = wx.StaticText( self, wx.ID_ANY, u"Glyph", wx.DefaultPosition, wx.Size( -1,-1 ), 0 )
		self.text_glyph.Wrap( -1 )

		sizer.Add( self.text_glyph, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_current = wx.StaticText( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 150,-1 ), wx.ST_ELLIPSIZE_MIDDLE|wx.BORDER_THEME )
		self.text_current.Wrap( -1 )

		sizer.Add( self.text_current, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_uni = wx.StaticText( self, wx.ID_ANY, u"Unicode", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.text_uni.Wrap( -1 )

		sizer.Add( self.text_uni, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_unicodes = wx.StaticText( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 100,-1 ), wx.ST_ELLIPSIZE_MIDDLE|wx.BORDER_THEME )
		self.text_unicodes.Wrap( -1 )

		sizer.Add( self.text_unicodes, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_selected = wx.StaticText( self, wx.ID_ANY, u"Selected", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.text_selected.Wrap( -1 )

		sizer.Add( self.text_selected, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_selected_count = wx.StaticText( self, wx.ID_ANY, u"0", wx.DefaultPosition, wx.Size( 35,-1 ), wx.ALIGN_RIGHT|wx.BORDER_THEME )
		self.text_selected_count.Wrap( -1 )

		sizer.Add( self.text_selected_count, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_of = wx.StaticText( self, wx.ID_ANY, u"/", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.text_of.Wrap( -1 )

		sizer.Add( self.text_of, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )

		self.text_total_count = wx.StaticText( self, wx.ID_ANY, u"0", wx.DefaultPosition, wx.Size( 35,-1 ), wx.ALIGN_RIGHT|wx.BORDER_THEME )
		self.text_total_count.Wrap( -1 )

		sizer.Add( self.text_total_count, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 2 )


		self.SetSizer( sizer )
		self.Layout()
		sizer.Fit( self )

		# Connect Events
		self.button_fontinfo.Bind( wx.EVT_BUTTON, self.on_button_fontinfo )
		self.text_current.Bind( wx.EVT_UPDATE_UI, self.update_current )
		self.text_unicodes.Bind( wx.EVT_UPDATE_UI, self.update_unicodes )
		self.text_selected_count.Bind( wx.EVT_UPDATE_UI, self.update_selected_count )
		self.text_total_count.Bind( wx.EVT_UPDATE_UI, self.update_total_count )

	def __del__( self ):
		pass


	# Virtual event handlers, override them in your derived class
	def on_button_fontinfo( self, event ):
		event.Skip()

	def update_current( self, event ):
		event.Skip()

	def update_unicodes( self, event ):
		event.Skip()

	def update_selected_count( self, event ):
		event.Skip()

	def update_total_count( self, event ):
		event.Skip()





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoNoteUI
###########################################################################

class FontInfoNoteUI ( wx.ScrolledWindow ):

	def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
		wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

		self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
		self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )

		sizerMain = wx.GridBagSizer( 0, 0 )
		sizerMain.SetFlexibleDirection( wx.BOTH )
		sizerMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.lbl_note = wx.StaticText( self, wx.ID_ANY, u"Note", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_note.Wrap( -1 )

		self.lbl_note.SetMinSize( wx.Size( 170,-1 ) )

		sizerMain.Add( self.lbl_note, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

		self.txtCtrl_note = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE, wx.DefaultValidator, u"note" )
		sizerMain.Add( self.txtCtrl_note, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND, 5 )


		sizerMain.AddGrowableCol( 1 )
		sizerMain.AddGrowableRow( 0 )

		self.SetSizer( sizerMain )
		self.Layout()
		sizerMain.Fit( self )

	def __del__( self ):
		pass





"""
layer
===============================================================================
"""
import logging
import time
import wx
from wx import aui

from ..dialog.assignLayerDialog import AssignLayerDialog
from ..dialog.layerDialog import getLayerInfo
from ..view.glyph import UfoGlyphView
from ..view.font import UfoFontView
from . import BaseToolbar

log = logging.getLogger(__name__)

copy_to, copy_from, exchange, clear, assign = range(5)


class LayerToolbar(BaseToolbar):
    def __init__(self, parent):
        super().__init__(parent, "GlyphShowToolbar")

        tool = self.appendTool(
            "Copy to Layer",
            "LAYER_COPY_TO",
            "Copy glyph from current to other layer",
            copy_to,
        )
        self.SetToolDropDown(tool.Id, True)
        self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.on_Tool, tool)

        tool = self.appendTool(
            "Copy from Layer",
            "LAYER_COPY_FROM",
            "Copy glyph from other layer to current layer",
            copy_from,
        )
        self.SetToolDropDown(tool.Id, True)
        self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.on_Tool, tool)

        tool = self.appendTool(
            "Exchange with Layer",
            "LAYER_EXCHANGE",
            "Exchange glyph with other layer",
            exchange,
        )
        self.SetToolDropDown(tool.Id, True)
        self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.on_Tool, tool)

        tool = self.appendTool(
            "Clear Layer",
            "LAYER_CLEAR",
            "Remove glyph from layer",
            clear,
        )
        self.SetToolDropDown(tool.Id, True)
        self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.on_Tool, tool)

        self.AddSeparator()

        tool = self.appendTool(
            "Assign Layer",
            "LAYER_ASSIGN",
            "Assign layer from font",
            assign,
        )
        # self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.on_Tool, tool)

    def _copy_glyph(self, glyphName, sourceLayerName, targetLayerName):
        view = self.currentView
        font = view.font
        if sourceLayerName not in font.layers:
            log.error("layer '%s' not found in font %r", sourceLayerName, font)
            return False
        sourceLayer = font.layers[sourceLayerName]
        if glyphName not in sourceLayer:
            log.warning(
                "glyph '%s' not found in layer '%s'", glyphName, sourceLayerName
            )
            return False
        sourceGlyph = sourceLayer[glyphName]
        if targetLayerName in font.layers:
            targetLayer = font.layers[targetLayerName]
        else:
            targetLayer = font.newLayer(targetLayerName)
        if glyphName in targetLayer:
            targetGlyph = targetLayer[glyphName]
        else:
            targetGlyph = targetLayer.newGlyph(glyphName)
        targetGlyph.copyDataFromGlyph(sourceGlyph)
        return True

    @staticmethod
    def _getNewLayerInfo(font):
        defaultName = "public.background"
        if defaultName in font.layers:
            defaultName = time.strftime("save %Y.%m.%d-%H:%M")
        return getLayerInfo(defaultName)

    def on_copy_to(self, event):
        targetLayerName = event.EventObject.FindItemById(event.Id).ItemLabelText
        view = self.currentView
        font = view.font
        targetLayerColor = None
        if targetLayerName == "<New Layer>":
            targetLayerName, targetLayerColor = self._getNewLayerInfo(font)
            if not targetLayerName:
                return
        if isinstance(view, UfoGlyphView):
            glyph = view.frame.canvas.glyph
            self._copy_glyph(glyph.name, glyph.layer.name, targetLayerName)
            planes = view.frame.canvas.getDrawingPlaneStack(targetLayerName)
            planes.visibleInactive = True
            view.frame.canvas.Refresh()
            view.frame.canvas.SetFocus()
        if isinstance(view, UfoFontView):
            for glyph in view.font.selectedGlyphs:
                self._copy_glyph(glyph.name, glyph.layer.name, targetLayerName)
        if targetLayerColor:
            font.layers[targetLayerName].color = targetLayerColor

    def on_copy_from(self, event):
        view = self.currentView
        sourceLayerName = event.EventObject.FindItemById(event.Id).ItemLabelText
        if isinstance(view, UfoGlyphView):
            glyph = view.frame.canvas.glyph
            glyph.undoManager.saveState()
            self._copy_glyph(glyph.name, sourceLayerName, glyph.layer.name)
            view.frame.canvas.Refresh()
            view.frame.canvas.SetFocus()
            return
        if isinstance(view, UfoFontView):
            for glyph in view.font.selectedGlyphs:
                self._copy_glyph(glyph.name, sourceLayerName, glyph.layer.name)

    def on_exchange(self, event):
        targetLayerName = event.EventObject.FindItemById(event.Id).ItemLabelText
        view = self.currentView
        font = view.font
        exchange_with_new = False
        targetLayerColor = None
        if targetLayerName == "<New Layer>":
            targetLayerName, targetLayerColor = self._getNewLayerInfo(font)
            if not targetLayerName:
                return
            exchange_with_new = True
        if not exchange_with_new:
            # move glyphs from target to temp layer
            temp_layer = time.strftime("__temp__%Y.%m.%d-%H:%M__")
            if isinstance(view, UfoGlyphView):
                glyph = view.frame.canvas.glyph
                self._copy_glyph(glyph.name, targetLayerName, temp_layer)
                font.layers[targetLayerName][glyph.name].clear()
            elif isinstance(view, UfoFontView):
                for glyphName in view.font.selectedGlyphNames:
                    self._copy_glyph(glyphName, targetLayerName, temp_layer)
                    font.layers[targetLayerName][glyphName].clear()
        # move glyphs from acvite to target layer
        if isinstance(view, UfoGlyphView):
            glyph = view.frame.canvas.glyph
            self._copy_glyph(glyph.name, glyph.layer.name, targetLayerName)
            glyph.clear()
        elif isinstance(view, UfoFontView):
            for glyph in view.font.selectedGlyphs:
                self._copy_glyph(
                    glyph.name, font.layers.defaultLayer.name, targetLayerName
                )
                glyph.clear()
        if not exchange_with_new:
            # move glyphs from temp to active layer
            if isinstance(view, UfoGlyphView):
                glyph = view.frame.canvas.glyph
                self._copy_glyph(glyph.name, temp_layer, glyph.layer.name)
                planes = view.frame.canvas.getDrawingPlaneStack(targetLayerName)
                planes.visibleInactive = True
                view.frame.canvas.Refresh()
                view.frame.canvas.SetFocus()
            elif isinstance(view, UfoFontView):
                for glyphName in view.font.selectedGlyphNames:
                    self._copy_glyph(
                        glyphName, temp_layer, font.layers.defaultLayer.name
                    )
            del font.layers[temp_layer]
        if targetLayerColor:
            font.layers[targetLayerName].color = targetLayerColor
        event.Skip()

    def on_clear(self, event):
        layerName = event.EventObject.FindItemById(event.Id).ItemLabelText
        view = self.currentView
        font = view.font
        layer = font.layers[layerName]
        if isinstance(view, UfoGlyphView):
            glyph = view.frame.canvas.glyph
            del layer[glyph.name]
            view.frame.canvas.Refresh()
            view.frame.canvas.SetFocus()
        if isinstance(view, UfoFontView):
            for glyphName in view.font.selectedGlyphNames:
                if glyphName in layer:
                    del layer[glyphName]
        if len(layer) == 0:
            del font.layers[layerName]

    def _on_assign(self, font):
        with AssignLayerDialog(self.currentView.frame) as dialog:
            dialog.target_font = font
            if dialog.ShowModal() == wx.ID_OK:
                targetLayerName = dialog.target_layer_name
                targetLayerColor = None
                if targetLayerName == "<New Layer>":
                    targetLayerName, targetLayerColor = self._getNewLayerInfo(font)
                    if not targetLayerName:
                        return
                sourceFont = dialog.source_font
                source_layer = sourceFont.layers[dialog.source_layer_name]
                target_font = dialog.target_font
                if targetLayerName in target_font.layers:
                    target_layer = target_font.layers[targetLayerName]
                else:
                    target_layer = target_font.newLayer(targetLayerName)
                for source_glyph in source_layer:
                    name = source_glyph.name
                    if name not in target_font:
                        if not dialog.create_new_glyphs:
                            continue
                        glyph = target_font.newGlyph(name)
                        glyph.unicodes = sourceFont[name].unicodes
                    if name in target_layer:
                        targetGlyph = target_layer[name]
                    else:
                        targetGlyph = target_layer.newGlyph(name)
                    targetGlyph.copyDataFromGlyph(source_glyph)
                if targetLayerColor:
                    font.layers[targetLayerName].color = targetLayerColor

    def on_Tool(self, event):
        self.SetToolSticky(event.Id, True)
        tool = self.FindTool(event.Id)
        i = tool.GetUserData()
        view = self.currentView
        if isinstance(view, UfoGlyphView):
            canvas = view.frame.canvas
            activeLayerName = canvas.getActiveDrawingPlaneStack().name
        elif isinstance(view, UfoFontView):
            activeLayerName = view.font.layers.defaultLayer.name
        if i == assign:
            self._on_assign(view.font)
            event.Skip()
            return
        menu = wx.Menu()
        if i == copy_to:
            for layer in view.font.layers:
                if layer.name != activeLayerName:
                    mnuItem = menu.Append(
                        wx.ID_ANY, layer.name, f"Copy to {layer.name}", wx.ITEM_NORMAL
                    )
                    self.Bind(wx.EVT_MENU, self.on_copy_to, mnuItem)
            menu.AppendSeparator()
            mnuItem = menu.Append(
                wx.ID_ANY, "<New Layer>", "Copy to New Layer", wx.ITEM_NORMAL
            )
            self.Bind(wx.EVT_MENU, self.on_copy_to, mnuItem)
        elif i == copy_from:
            for layer in view.font.layers:
                if layer.name != activeLayerName:
                    mnuItem = menu.Append(
                        wx.ID_ANY, layer.name, f"Copy from {layer.name}", wx.ITEM_NORMAL
                    )
                    self.Bind(wx.EVT_MENU, self.on_copy_from, mnuItem)
        elif i == exchange:
            for layer in view.font.layers:
                if layer.name != activeLayerName:
                    mnuItem = menu.Append(
                        wx.ID_ANY,
                        layer.name,
                        f"Exchange with {layer.name}",
                        wx.ITEM_NORMAL,
                    )
                    self.Bind(wx.EVT_MENU, self.on_exchange, mnuItem)
            menu.AppendSeparator()
            mnuItem = menu.Append(
                wx.ID_ANY, "<New Layer>", "Exchange with New Layer", wx.ITEM_NORMAL
            )
            self.Bind(wx.EVT_MENU, self.on_exchange, mnuItem)
        elif i == clear:
            for layer in view.font.layers:
                if layer.name not in (
                    activeLayerName,
                    view.font.layers.defaultLayer.name,
                ):
                    mnuItem = menu.Append(
                        wx.ID_ANY, layer.name, f"Clear {layer.name}", wx.ITEM_NORMAL
                    )
                    self.Bind(wx.EVT_MENU, self.on_clear, mnuItem)

        self.SetToolSticky(event.Id, False)
        self.PopupMenu(menu)
        menu.Destroy()
        event.Skip()

    def on_update_Tool(self, event):
        tool = self.FindTool(event.Id)
        i = tool.GetUserData()
        view = self.currentView
        if isinstance(view, UfoGlyphView):
            if i in (copy_from, clear):
                event.Enable(len(view.font.layers) > 1)
            else:
                event.Enable(True)
        elif isinstance(view, UfoFontView):
            if view.font is not None and view.font.selectedGlyphCount:
                if i in (copy_from, clear):
                    event.Enable(len(view.font.layers) > 1)
                else:
                    event.Enable(True)
            else:
                event.Enable(False)
        else:
            event.Enable(False)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class ComponentDialogUI
###########################################################################

class ComponentDialogUI ( wx.Dialog ):

	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Component", pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.CAPTION|wx.CLOSE_BOX|wx.RESIZE_BORDER|wx.STAY_ON_TOP )

		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

		sizerMain = wx.BoxSizer( wx.VERTICAL )

		sizer_name = wx.BoxSizer( wx.HORIZONTAL )

		self.label_name = wx.StaticText( self, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_name.Wrap( -1 )

		sizer_name.Add( self.label_name, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.textCtrl_name = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		sizer_name.Add( self.textCtrl_name, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.button_name = wx.Button( self, wx.ID_ANY, u"...", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
		sizer_name.Add( self.button_name, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizerMain.Add( sizer_name, 0, wx.ALL|wx.EXPAND, 5 )

		sizerPos = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Position [Unit]" ), wx.HORIZONTAL )

		self.label_pos_x = wx.StaticText( sizerPos.GetStaticBox(), wx.ID_ANY, u"x", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_pos_x.Wrap( -1 )

		sizerPos.Add( self.label_pos_x, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_pos_x = wx.SpinCtrl( sizerPos.GetStaticBox(), wx.ID_ANY, u"0", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, -65535, 65535, 0 )
		sizerPos.Add( self.spinCtrl_pos_x, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.label_pos_y = wx.StaticText( sizerPos.GetStaticBox(), wx.ID_ANY, u"y", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_pos_y.Wrap( -1 )

		sizerPos.Add( self.label_pos_y, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_pos_y = wx.SpinCtrl( sizerPos.GetStaticBox(), wx.ID_ANY, u"0", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, -65535, 65535, 0 )
		sizerPos.Add( self.spinCtrl_pos_y, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizerMain.Add( sizerPos, 0, wx.ALL|wx.EXPAND, 5 )

		sizerScale = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Scale [%]" ), wx.HORIZONTAL )

		self.label_scale_x = wx.StaticText( sizerScale.GetStaticBox(), wx.ID_ANY, u"x", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_scale_x.Wrap( -1 )

		sizerScale.Add( self.label_scale_x, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_scale_x = wx.SpinCtrl( sizerScale.GetStaticBox(), wx.ID_ANY, u"100", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, -1000, 1000, 100 )
		sizerScale.Add( self.spinCtrl_scale_x, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.label_scale_y = wx.StaticText( sizerScale.GetStaticBox(), wx.ID_ANY, u"y", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_scale_y.Wrap( -1 )

		sizerScale.Add( self.label_scale_y, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_scale_y = wx.SpinCtrl( sizerScale.GetStaticBox(), wx.ID_ANY, u"100", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, -1000, 1000, 100 )
		sizerScale.Add( self.spinCtrl_scale_y, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizerMain.Add( sizerScale, 0, wx.ALL|wx.EXPAND, 5 )

		sizerDlgButtons = wx.StdDialogButtonSizer()
		self.sizerDlgButtonsOK = wx.Button( self, wx.ID_OK )
		sizerDlgButtons.AddButton( self.sizerDlgButtonsOK )
		self.sizerDlgButtonsCancel = wx.Button( self, wx.ID_CANCEL )
		sizerDlgButtons.AddButton( self.sizerDlgButtonsCancel )
		sizerDlgButtons.Realize();

		sizerMain.Add( sizerDlgButtons, 0, wx.ALL|wx.EXPAND, 5 )


		self.SetSizer( sizerMain )
		self.Layout()
		sizerMain.Fit( self )

		self.Centre( wx.BOTH )

		# Connect Events
		self.button_name.Bind( wx.EVT_BUTTON, self.on_button_name )

	def __del__( self ):
		pass


	# Virtual event handlers, override them in your derived class
	def on_button_name( self, event ):
		event.Skip()





"""
toolmeter
===============================================================================

Impementation of the Meter tool for the glyph editor
"""
import wx

from wbDefcon.pens.lineIntersectionPen import LineIntersectionPen
from wbBase.tools import get_wxBrush, get_wxFont, get_wxPen

from ..cursor import cursors

from .base import WORKING, GlyphTool


class ToolMeter(GlyphTool):
    """
    Tool to mesure distances between intersection points with a drawn line.
    """
    cursor = cursors["Meter"]
    pen = get_wxPen(wx.Colour(128, 128, 150, 220))
    brush = get_wxBrush(wx.Colour(128, 128, 150, 64))
    labelfont = get_wxFont(pointSize=7, faceName="Small Fonts")
    labelpen = wx.TRANSPARENT_PEN
    labelbrush = get_wxBrush(wx.Colour(230, 230, 240, 100))

    def __init__(self, parent):
        super().__init__(parent)
        self.lineIntersectionPen = LineIntersectionPen()
        self.meterStartPoint = None
        self.meterEndPoint = None

    def reset(self):
        super().reset()
        self.lineIntersectionPen.reset()
        self.meterStartPoint = None
        self.meterEndPoint = None

    @property
    def meterLine(self):
        if self.meterStartPoint and self.meterEndPoint:
            return (self.meterStartPoint, self.meterEndPoint)
        return None

    @property
    def intersections(self):
        glyph = self.glyph
        if glyph is None:
            return []
        line = self.meterLine
        if not line:
            return []
        pen = self.lineIntersectionPen
        if line != pen.line:
            pen.line = line
            glyph.draw(pen)
        return list(pen.intersections)

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_LEFT_DOWN(self, event):
        super().on_LEFT_DOWN(event)
        glyph = self.glyph
        if glyph is not None:
            self.lineIntersectionPen.glyphSet = glyph.layer
            self.meterStartPoint = (
                round(self.canvas.screenToCanvasX(self.x)),
                round(self.canvas.screenToCanvasY(self.y)),
            )
            self.state = WORKING
        event.Skip()

    def on_MOTION(self, event):
        super().on_MOTION(event)
        if event.Dragging() and self.state == WORKING:
            x = self.x
            y = self.y
            if self.shiftDown:
                dx = abs(self.xClick - self.x)
                dy = abs(self.yClick - self.y)
                if dx > dy:
                    y = self.yClick
                else:
                    x = self.xClick
            self.meterEndPoint = (
                round(self.canvas.screenToCanvasX(x)),
                round(self.canvas.screenToCanvasY(y)),
            )
            self.canvas.Refresh()
        event.Skip()

    def draw(self, gc):
        if self.state == WORKING:
            gc.SetFont(self.labelfont, wx.BLACK)
            transformPoint = self.canvas.transform.transformPoint
            start = transformPoint(self.meterStartPoint)
            end = transformPoint(self.meterEndPoint)
            gc.SetPen(self.pen)
            gc.StrokeLine(start[0], start[1], end[0], end[1])
            d = 3
            s = 3
            points = list(self.intersections)
            points.append(self.meterStartPoint)
            points.append(self.meterEndPoint)
            points.sort()
            xPrev = None
            yPrev = None
            for i, intersection in enumerate(points):
                if intersection:
                    try:
                        x, y = transformPoint(intersection)
                        x = round(x)
                        y = round(y)
                    except ValueError:
                        continue
                    gc.SetPen(self.pen)
                    gc.SetBrush(self.brush)
                    gc.DrawEllipse(x - s, y - s, 2 * s, 2 * s)
                    text = "%.1f / %.1f" % (intersection[0], intersection[1])
                    w, h = gc.GetTextExtent(text)
                    w = int(round(w)) + 2 * s
                    h = int(round(h)) + 2 * s
                    gc.SetPen(self.labelpen)
                    gc.SetBrush(self.labelbrush)
                    gc.DrawRoundedRectangle(x + d, y + d, w, h, d)
                    gc.DrawText(text, x + d + s, y + d + s - 1)
                    if i > 0 and points[i - 1]:
                        try:
                            dist = self.distance(points[i - 1], intersection)
                        except ValueError:
                            dist = 0
                        if dist > 0.5:
                            text = "%.1f" % dist
                            w, h = gc.GetTextExtent(text)
                            w = int(round(w)) + 2 * s
                            h = int(round(h)) + 2 * s
                            xDist = int(round(xPrev + (x - xPrev) * 0.5))
                            yDist = int(round(yPrev + (y - yPrev) * 0.5))
                            gc.DrawRoundedRectangle(xDist - w - d, yDist - d - h, w, h, d)
                            gc.DrawText(text, xDist - w - d + s, yDist - d - h + s - 1)
                    xPrev = x
                    yPrev = y
